/******************************************************************************
                          C++ Interface: IVisualization
                            -------------------
    description       :
    begin             : Thu Apr 21 2005
    author            : Martin Marinov, (C) 2005
    email             : marinov@cs.rwth-aachen.de
    copyright         : See LICENSE.txt file that comes with this distribution
 ******************************************************************************/

#ifndef __IVisualization_h__
#define __IVisualization_h__

#include <map>

struct IVisualization
{
  enum ModeEnum { MODE_POINTS, MODE_WIREFRAME, MODE_SOLID };
  
  virtual ~IVisualization() {};
  
  virtual bool                              init() = 0;
  virtual void                              draw(ModeEnum _mode) = 0;
  virtual void                              dump() = 0;
};

struct TwoScaleModel;

typedef IVisualization* (*VisualizationProducer)(const TwoScaleModel& _tsm, int argc, char* argv[]) ;
typedef std::map<char, VisualizationProducer> KeyVisualizationMap;

struct VisualizationFactory
{
private:
  VisualizationFactory();
  VisualizationFactory(const VisualizationFactory&);
  VisualizationFactory& operator = (const VisualizationFactory&);
  
  static KeyVisualizationMap&               instance();
  
public:
  static const KeyVisualizationMap&         map();
  static void                               connect(char _key, VisualizationProducer _vp_f);
}; 

#endif//__IVisualization_h__
