/******************************************************************************
                       C++ Implementation: OglHelpers
                            -------------------
    description       :
    begin             : Thu Mar 24 2005
    author            : Martin Marinov, (C) 2005
    email             : marinov@cs.rwth-aachen.de
    copyright         : See LICENSE.txt file that comes with this distribution
 ******************************************************************************/

#include "OglHelpers.h"

#ifdef WIN32
	#include <windows.h>
#endif

#ifdef WIN32
	#include <windows.h>
	#include <GL/glew.h>
#else
	#include <GL/gl.h>
	#include <GL/glu.h>
	#define GL_GLEXT_PROTOTYPES
	#include <GL/glext.h>
#endif

#include <GL/glut.h>

#include "defs.h"
#include "TwoScaleModel.h"

void check_ogl_error()
{
  GLenum err_code = glGetError();
  switch (err_code) 
  {
  case GL_NO_ERROR          : break;
  case GL_INVALID_ENUM      : CHECK_FALSE; break;                  
  case GL_INVALID_VALUE     : CHECK_FALSE; break;
  case GL_INVALID_OPERATION : CHECK_FALSE; break;
  case GL_STACK_OVERFLOW    : CHECK_FALSE; break;
  case GL_STACK_UNDERFLOW   : CHECK_FALSE; break;
  case GL_OUT_OF_MEMORY     : CHECK_FALSE; break;
  default:
    CHECK_FALSE;//unknown error?
  }
}

void draw_grid(int plane)
{
  #define OGL_GRID_LIMIT 1000 
  double lb = -OGL_GRID_LIMIT;
  double hb = OGL_GRID_LIMIT;
  uint n_side_lines = 21;
//   uint n_total_lines = n_side_lines*n_side_lines;
  Vec3 corner[4];
  switch(plane)
  {
  case 0  :
    corner[0] = Vec3(lb, lb, 0);
    corner[1] = Vec3(lb, hb, 0);
    corner[2] = Vec3(hb, hb, 0);
    corner[3] = Vec3(hb, lb, 0);
    break;
  case 1  :
    corner[0] = Vec3(lb, 0, lb);
    corner[1] = Vec3(lb, 0, hb);
    corner[2] = Vec3(hb, 0, hb);
    corner[3] = Vec3(hb, 0, lb);
    break;
  case 2 :
    corner[0] = Vec3(0, lb, lb);
    corner[1] = Vec3(0, lb, hb);
    corner[2] = Vec3(0, hb, hb);
    corner[3] = Vec3(0, hb, lb);
    break;
  }
  glDisable(GL_LIGHTING);
  glBegin(GL_LINES);
  double step = 1.0/(n_side_lines - 1);
  for (uint i = 0; i < n_side_lines; ++i)
  {
    double t1 = i*step;
    double t0 = 1.0 - t1;
    //set horizontal line
    Vec3 pt0(0), pt1(0), pt2(0), pt3(0);;
    pt0.add_linear(t0, corner[0]);
    pt0.add_linear(t1, corner[1]);
    pt1.add_linear(t0, corner[3]);
    pt1.add_linear(t1, corner[2]);
    glVertex3f(pt0.x, pt0.y, pt0.z);
    glVertex3f(pt1.x, pt1.y, pt1.z);
    //set vertical line
    pt2.add_linear(t0, corner[0]);
    pt2.add_linear(t1, corner[3]);
    pt3.add_linear(t0, corner[1]);
    pt3.add_linear(t1, corner[2]);
    glVertex3f(pt2.x, pt2.y, pt2.z);
    glVertex3f(pt3.x, pt3.y, pt3.z);
  }
  glEnd();
  glEnable(GL_LIGHTING);
}

/*  
  glBegin(GL_LINE_LOOP);
  for (uint i = 0; i < ts_model__.cm_vertices_.size(); ++i)
  {
    glVertex3f(ts_model__.cm_vertices_[i].x, ts_model__.cm_vertices_[i].y, 
               ts_model__.cm_vertices_[i].z);
  }
  glEnd(); 
  */
  
// void draw_ts_mid_pt()
// {
//   Vec3 mid_pt;
//   ts_model__.cm_bb_.calc_mid_pt(mid_pt);
//   glPointSize(50);
//   
//   glBegin(GL_POINTS);
//     glVertex3fv(mid_pt.coords);
//   glEnd();
//   
//   glPointSize(1);
// }
