/******************************************************************************
                          C++ Interface: TSMVisualVP
                            -------------------
    description       :
    begin             : Wed Mar 23 2005
    author            : Martin Marinov, (C) 2005
    email             : marinov@cs.rwth-aachen.de
    copyright         : See LICENSE.txt file that comes with this distribution
 ******************************************************************************/

#ifndef __TSMVisualARB_h__
#define __TSMVisualARB_h__

#include <GL/glew.h>
#include <vector>
#include <string>
#include "defs.h"
#include "IVisualization.h"
#include "globjects.hh"
#include "LightMap.hh"


//#define CUSTOM_PREC_ATTRIBS
//#define SEPARATE_PREC_ATTRIBS

struct TwoScaleModel;

class TSMVisualARB : public IVisualization
{
public:  
  typedef float                             Scalar;
    
private:

  const TwoScaleModel&                      tsm_;
  std::string                               vp_path_;
  VertexProgramARB                          vertex_program_;
  LightMap                                  light_map_;
  GeometryBuffer                            vbo_;
  IndexBuffer                               ibo_;
  uint                                      n_indices_;  
  std::vector<uint>                         modified_cvs_;
  
private:

  bool                                      load_vertex_program(const char* _filename);
  void                                      unload_vertex_program();
  void                                      activate();
  void                                      deactivate();
  

public:

  TSMVisualARB(const TwoScaleModel& _tsm, const char* _vp_path);
  virtual ~TSMVisualARB();
  
  /// IVisualization API
  virtual bool                              init();
  virtual void                              draw(ModeEnum _mode);
  virtual void                              dump();
};


#endif//__TSMVisualARB_h__
