/******************************************************************************
                          C++ Interface: TSMVisualVBO
                            -------------------
    description       :
    begin             : Wed Mar 23 2005
    author            : Martin Marinov, (C) 2005
    email             : marinov@cs.rwth-aachen.de
    copyright         : See LICENSE.txt file that comes with this distribution
 ******************************************************************************/

#ifndef __TSMVisualVBO_h__
#define __TSMVisualVBO_h__

#define PRECOMPUTE_STATIC_FRAC

#include "defs.h"
#include "IVisualization.h"
#include "TwoScaleModel.h"

class TSMVisualVBO : public IVisualization
{
private:  
  typedef float                             Scalar;
  typedef TwoScaleModel::CVCoeffArray       CVCoeffArray;
  struct FSVGeometry 
  {
    Vec3 pt_, n_;
  };
  
private:
  const TwoScaleModel&                      tsm_;
  
#ifdef PRECOMPUTE_STATIC_FRAC  
  std::vector<uint>                         modified_cvs_;
  char*                                     fsv_blocks_;
  uint                                      vbo_fsv_geom_;
  FSVGeometry*                              fsv_geom_;
#else
  uint                                      vbo_vertices_;
  uint                                      vbo_normals_;
  float*                                    vertices_;
  float*                                    normals_;
#endif//PRECOMPUTE_STATIC_FRAC
  uint                                      vbo_indices_;
  uint                                      n_indices_;  

private:
  bool                                      map_geometry_vbos();
  bool                                      unmap_geometry_vbos();
  void                                      update_geometry();
  
public:
  TSMVisualVBO(const TwoScaleModel& _tsm);
  virtual ~TSMVisualVBO();
  
  /// IVisualization API
  virtual bool                              init();
  virtual void                              draw(ModeEnum _mode);
  virtual void                              dump();
};

#endif//__TSMVisualVBO_h__
