/******************************************************************************
                          C++ Interface: TSMVisualVP
                            -------------------
    description       :
    begin             : Wed Mar 23 2005
    author            : Martin Marinov, (C) 2005
    email             : marinov@cs.rwth-aachen.de
    copyright         : See LICENSE.txt file that comes with this distribution
 ******************************************************************************/

#ifndef __TSMVisualVP_h__
#define __TSMVisualVP_h__

#include <GL/glew.h>
#include <vector>
#include <string>
#include "defs.h"
#include "IVisualization.h"

//#define CUSTOM_PREC_ATTRIBS
//#define SEPARATE_PREC_ATTRIBS

class VertexProgram;
struct TwoScaleModel;

class TSMVisualVP : public IVisualization
{
public:  
  typedef float                             Scalar;
    
private:
  const TwoScaleModel&                      tsm_;
  std::string                               vs_file_name_;
  VertexProgram*                            vertex_program_;
#ifdef SEPARATE_PREC_ATTRIBS
  uint                                      vbo_vertices_;
  uint                                      vbo_prec_pt_;
  uint                                      vbo_prec_du_;
  uint                                      vbo_prec_dv_;
#else
  uint                                      vbo_fsv_blocks_;
#endif
  
  int                                       loc_prec_foot_pt_;
  int                                       loc_prec_foot_du_;
  int                                       loc_prec_foot_dv_;
  
  uint                                      n_indices_;  
  uint                                      vbo_indices_;
  std::vector<uint>                         modified_cvs_;
  std::vector<int>                          loc_cvs_;//uniforms' locations
  std::vector<uint>                         vbo_attribs_;
  std::vector<int>                          loc_attribs_;//attributes' locations
  
private:
  bool                                      load_vertex_program();
  void                                      unload_vertex_program();
  
  void                                      activate();
  void                                      deactivate();
  
public:
  TSMVisualVP(const TwoScaleModel& _tsm, const std::string& _vs_file_name);
  virtual ~TSMVisualVP();
  
  /// IVisualization API
  virtual bool                              init();
  virtual void                              draw(ModeEnum _mode);
  virtual void                              dump();
};


#endif//__TSMVisualVP_h__
