/******************************************************************************
                          C++ Interface: TSMVisualVT
                            -------------------
    description       :
    begin             : Wed Apr 20 2005
    author            : Martin Marinov, (C) 2005
    email             : marinov@cs.rwth-aachen.de
    copyright         : See LICENSE.txt file that comes with this distribution
 ******************************************************************************/

#ifndef __TSMVisualVT_h__
#define __TSMVisualVT_h__

#include <GL/glew.h>
#include <vector>
#include <string>
#include "defs.h"
#include "IVisualization.h"

class VertexProgram;
struct TwoScaleModel;
struct Vec3Wt;

class TSMVisualVT : public IVisualization
{
public:  
  typedef float                             Scalar;
    
private:
  const TwoScaleModel&                      tsm_;
  std::string                               vs_file_name_;
  VertexProgram*                            vertex_program_;

  /// fully static VBOs - these don't change at all for a certain TwoScaleModel
  uint                                      vbo_vertices_;
  uint                                      vbo_normals_;
  uint                                      n_indices_;  
  uint                                      vbo_indices_;
  /// (pseudo)tex coords inside tex_coeff_
  uint                                      vbo_tex_coords_;
  /// Coefficients texture - fully static
  uint                                      tex_coeffs_;
  int                                       tex_coeffs_w_;
  int                                       tex_coeffs_h_;
  int                                       loc_tex_coeffs_;
  int                                       loc_tex_coeffs_w_;
  int                                       loc_tex_coeffs_h_;

  /// Control vertices texture - dynamic (changes every frame)
  uint                                      tex_cvs_;
  int                                       tex_cvs_w_;
  int                                       tex_cvs_h_;
  std::vector<Vec3Wt>                       tex_cvs_buf_;
  int                                       loc_tex_cvs_;
  int                                       loc_tex_cvs_w_;
  int                                       loc_tex_cvs_h_;
/*
  //test
  uint                                      tex_vertices_;
  uint                                      tex_normals_;
  int                                       tex_w_;
  int                                       tex_h_;

  uint                                      tex_vertices_loc_; 
  uint                                      tex_normals_loc_;
  uint                                      tex_w_loc_;
  uint                                      tex_h_loc_;
 */
private:
  void                                      unload_vertex_program();
  bool                                      init_data();
  bool                                      init_tex_coeffs();
  bool                                      init_tex_cvs();
  void                                      activate();  
  void                                      deactivate();  
  
public:
  TSMVisualVT(const TwoScaleModel& _tsm, const std::string& _vs_file_name);
  virtual ~TSMVisualVT();
  
  /// IVisualization API
  virtual bool                              init();
  virtual void                              draw(ModeEnum _mode);
  virtual void                              dump();
};

#endif//__TSMVisualVT_h__
