/******************************************************************************
                           C++ Interface: Vector
                            -------------------
    description       :
    begin             : Fri Apr 22 2005
    author            : Martin Marinov, (C) 2005
    email             : marinov@cs.rwth-aachen.de
    copyright         : See LICENSE.txt file that comes with this distribution
 ******************************************************************************/

#ifndef __Vector_h__
#define __Vector_h__

#include <math.h>
#include "defs.h"

struct Vec3
{
  typedef float Scalar;
  union
  {
    struct { Scalar x, y, z; };
    Scalar coords[3];
  };
  
  inline Vec3() {}
  inline Vec3(Scalar _t) : x(_t), y(_t), z(_t) {}
  inline Vec3(Scalar _x, Scalar _y, Scalar _z) : x(_x), y(_y), z(_z)  {}
  
  inline Scalar& operator [] (int i) { CHECK(i < 3); return coords[i]; }
  inline Scalar  operator [] (int i) const { CHECK(i < 3); return coords[i]; }
  
  inline void operator += (const Vec3& _a) { x += _a.x; y += _a.y; z += _a.z; }
  inline void operator -= (const Vec3& _a) { x -= _a.x; y -= _a.y; z -= _a.z; }
  inline void operator *= (Scalar t) { x *= t; y *= t; z *= t; }
  inline void operator /= (Scalar t) { x /= t; y /= t; z /= t; }
  inline Scalar operator * (const Vec3& _a) const  { return x*_a.x + y*_a.y + z*_a.z; }
  
  inline void set_linear(Scalar _t, const Vec3& _a) 
  { 
    x = _t*_a.x; 
    y = _t*_a.y; 
    z = _t*_a.z; 
  }
  inline void add_linear(Scalar _t, const Vec3& _a) 
  { 
    x += _t*_a.x; 
    y += _t*_a.y; 
    z += _t*_a.z; 
  }
  inline void set_sum(const Vec3& _a, const Vec3& _b) 
  { 
    x = _a.x + _b.x;
    y = _a.y + _b.y;
    z = _a.z + _b.z;
  }
  inline void set_cross(const Vec3& _a, const Vec3& _b) 
  {                                       
    x = _a[1]*_b[2] - _a[2]*_b[1];     
    y = _a[2]*_b[0] - _a[0]*_b[2];     
    z = _a[0]*_b[1] - _a[1]*_b[0];     
  }
  inline Scalar norm() const { return (Scalar)sqrt(sqrnorm()); }
  inline Scalar sqrnorm() const { return x*x + y*y + z*z; }
  inline void normalize(){ (*this) /= norm(); };
};

struct Vec3Wt
{
  Vec3 v;
  float w;
};

struct Vec2i
{
  typedef int                               Scalar;
  union
  {
    struct { Scalar x, y; };
    Scalar coords[2];
  };
  
  inline Vec2i() {}
  inline Vec2i(Scalar _t) : x(_t), y(_t) {}
  inline Vec2i(Scalar _x, Scalar _y) : x(_x), y(_y)  {}
  
  inline Scalar& operator [] (int i) { CHECK(i < 2); return coords[i]; }
  inline Scalar  operator [] (int i) const { CHECK(i < 2); return coords[i]; }
};

struct Vec4
{
  union
  {
    struct { float x, y, z, w; };
    float coords[4];
  };
  
  inline float  operator [] (uint i) const { CHECK(i < 4); return coords[i]; }
  inline float& operator [] (uint i)       { CHECK(i < 4); return coords[i]; }
  inline const Vec3&  v() const { return reinterpret_cast<const Vec3&>(*this); }
  inline       Vec3&  v()       { return reinterpret_cast<Vec3&>(*this); }
};

#endif//__Vector_h__
