/******************************************************************************
                          C++ Interface: vbo
                            -------------------
    description       :
    begin             : Wed Apr 20 2005
    author            : Martin Marinov, (C) 2005
    email             : marinov@cs.rwth-aachen.de
    copyright         : See LICENSE.txt file that comes with this distribution
 ******************************************************************************/

#ifndef __vbo_h__
#define __vbo_h__

#include <vector>
#ifdef WIN32
  #include <windows.h>
#endif
#include <GL/glew.h>
#include "defs.h"

template <class T>
__inline__ uint new_VBO(const std::vector<T>& _array, GLenum _vbo_type = GL_ARRAY_BUFFER_ARB, 
                                                      GLenum _vbo_usage = GL_STATIC_DRAW_ARB)
{
  uint vbo_id;
  uint data_size = _array.size()*sizeof(T);
  OGL_CHECK(::glGenBuffersARB(1, &vbo_id));
  OGL_CHECK(::glBindBufferARB(_vbo_type, vbo_id));//bind vertex array
  OGL_CHECK(::glBufferDataARB(_vbo_type, data_size, &_array[0], _vbo_usage));
  OGL_CHECK(::glBindBufferARB(_vbo_type, 0));//unbind the array buffer
  return vbo_id;
}

__inline__ void delete_VBO(uint _handle)
{ OGL_CHECK(::glDeleteBuffersARB(1, &_handle)); }

__inline__ void delete_VBOs(std::vector<uint>& _vbos)
{ 
  if (!_vbos.empty())
  {
    OGL_CHECK(::glDeleteBuffersARB(_vbos.size(), &_vbos[0]));
  } 
}

#endif//__vbo_h__
